from abc import abstractmethod
from typing import Callable

from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.server import Server
from modules.stats.model.statistic_result import StatisticResult


class StatisticService:

    def __init__(
            self,
            auto_roles: AutoRoleRepositoryABC,
            clients: ClientRepositoryABC,
            known_users: KnownUserRepositoryABC,
            levels: LevelRepositoryABC,
            servers: ServerRepositoryABC,
            user_joined_servers: UserJoinedServerRepositoryABC,
            user_joined_voice_channel: UserJoinedVoiceChannelRepositoryABC,
            users: UserRepositoryABC,
            bot: DiscordBotServiceABC,
    ):
        self._auto_roles = auto_roles
        self._clients = clients
        self._known_users = known_users
        self._levels = levels
        self._servers = servers
        self._user_joined_servers = user_joined_servers
        self._user_joined_voice_channel = user_joined_voice_channel
        self._users = users
        self._bot = bot

    async def execute(self, _f: Callable, server: Server) -> StatisticResult:
        guild = self._bot.guilds.where(lambda g: g.id == server.discord_server_id).single()

        return await _f(
            self._auto_roles
                .get_auto_roles()
                .where(lambda x: x.server.server_id == server.server_id),
            self._clients
                .get_clients()
                .where(lambda x: x.server.server_id == server.server_id),
            self._known_users.get_users(),
            self._levels
                .get_levels()
                .where(lambda x: x.server.server_id == server.server_id),
            self._servers
                .get_servers()
                .where(lambda x: x.server_id == server.server_id),
            self._user_joined_servers
                .get_user_joined_servers()
                .where(lambda x: x.user.server.server_id == server.server_id),
            self._user_joined_voice_channel
                .get_user_joined_voice_channels()
                .where(lambda x: x.user.server.server_id == server.server_id),
            self._users
                .get_users()
                .where(lambda x: x.server.server_id == server.server_id),
            guild
        )
